@@ -120,10 +120,12 @@ void IOPortManager::exportXml(ticpp::Element* pConfig)
 }
 
 IOPort::IOPort()
+	: rxThread_m(NULL)
 {}
 
 IOPort::~IOPort()
 {
+	delete rxThread_m;
 }
 
 IOPort* IOPort::create(const std::string& type)
@@ -156,7 +158,10 @@ void IOPort::importXml(ticpp::Element* pConfig)
 {
     id_m = pConfig->GetAttribute("id");
     if (isRxEnabled())
-        rxThread_m.reset(new RxThread(this));
+	{
+		delete rxThread_m;
+        rxThread_m = new RxThread(this);
+	}
 }
 
 void IOPort::exportXml(ticpp::Element* pConfig)
@@ -166,13 +171,13 @@ void IOPort::exportXml(ticpp::Element* pConfig)
 
 void IOPort::addListener(IOPortListener *l)
 {
-    if (rxThread_m.get())
+    if (rxThread_m)
         rxThread_m->addListener(l);
 }
 
 bool IOPort::removeListener(IOPortListener *l)
 {
-    if (rxThread_m.get())
+    if (rxThread_m)
         return (rxThread_m->removeListener(l));
     else
         return false;
@@ -351,7 +356,9 @@ TcpClientIOPort::Socket::Socket(TcpClientIOPort *ioport)
         if (sockfd_m >= 0) {
             if (pth_connect(sockfd_m, (const struct sockaddr *)&ioport->addr_m, sizeof (ioport_m->addr_m)) < 0) {
                 logger_m.errorStream() << "Unable to connect to server for ioport " << ioport->getID() << endlog;
-            }
+            } else { 
+        	ioport->onConnect(); 
+    	    }
         }
         else {
             logger_m.errorStream() << "Unable to create  socket for ioport " << ioport_m->getID() << endlog;
@@ -1080,6 +1087,14 @@ void ConnectCondition::importXml(ticpp::Element* pConfig)
         msg << "ConnectCondition: Only serial and TCP port can use connectCondition" << std::endl;
         throw ticpp::Exception(msg.str());
     }
+    
+    TcpClientIOPort* tcpPort = dynamic_cast<TcpClientIOPort*>(port);
+    if ((tcpPort) && (!tcpPort->isRxEnabled()))
+    {
+        std::stringstream msg;
+        msg << "ConnectCondition: TCP port must be permanent to use connectCondition" << std::endl;
+        throw ticpp::Exception(msg.str());
+    }
 
     port->addConnectListener(this);
 }
