@@ -23,6 +23,7 @@
 #include <cmath>
 #include <cassert>
 #include <iomanip>
+#include <iconv.h>
 
 ObjectController* ObjectController::instance_m;
 
@@ -191,6 +192,8 @@ Object* Object::create(const std::string& type)
         return new StringObject();
     else if (type == "232.600")
         return new RGBObject();
+    else if (type == "251.600")
+        return new RGBWObject();
     else
         return 0;
 }
@@ -420,6 +423,12 @@ void Object::exportXml(ticpp::Element* pConfig)
 void Object::read()
 {
     KnxConnection* con = Services::instance()->getKnxConnection();
+	if (con->isVoid())
+	{
+		init_m = true;
+		return;
+	}
+
     if (!readPending_m)
     {
         uint8_t buf[2] = { 0, 0 };
@@ -656,7 +665,7 @@ void SwitchingObject::doWrite(const uint8_t* buf, int len, eibaddr_t src)
 
 void SwitchingObject::doSend(bool isWrite)
 {
-    uint8_t buf[2] = { 0, (isWrite ? 0x80 : 0x40) | (getBoolObjectValue() ? 1 : 0) };
+    uint8_t buf[2] = { 0, (uint8_t)((isWrite ? 0x80 : 0x40) | (getBoolObjectValue() ? 1 : 0)) };
     Services::instance()->getKnxConnection()->write(getGad(), buf, 2);
 }
 
@@ -864,7 +873,7 @@ void StepDirObject::doWrite(const uint8_t* buf, int len, eibaddr_t src)
 
 void StepDirObject::doSend(bool isWrite)
 {
-    uint8_t buf[2] = { 0, (isWrite ? 0x80 : 0x40) | (getDirection() ? 8 : 0) | (getStepCode() & 0x07) };
+    uint8_t buf[2] = { 0, (uint8_t)((isWrite ? 0x80 : 0x40) | (getDirection() ? 8 : 0) | (getStepCode() & 0x07)) };
     Services::instance()->getKnxConnection()->write(getGad(), buf, 2);
 }
 
@@ -1231,7 +1240,7 @@ void TimeObject::setTime(time_t time)
 
 void TimeObject::doSend(bool isWrite)
 {
-    uint8_t buf[5] = { 0, (isWrite ? 0x80 : 0x40), ((wday_m<<5) & 0xE0) | (hour_m & 0x1F), min_m, sec_m };
+    uint8_t buf[5] = { 0, (uint8_t)(isWrite ? 0x80 : 0x40), (uint8_t)(((wday_m<<5) & 0xE0) | (hour_m & 0x1F)), (uint8_t)min_m, (uint8_t)sec_m };
     Services::instance()->getKnxConnection()->write(getGad(), buf, 5);
 }
 
@@ -1444,11 +1453,11 @@ void DateObject::setDate(time_t time)
 
 void DateObject::doSend(bool isWrite)
 {
-    uint8_t buf[5] = { 0,
-                       (isWrite ? 0x80 : 0x40),
-                       day_m, month_m,
-                       (year_m >= 100 && year_m < 190) ? year_m-100 : year_m };
-    Services::instance()->getKnxConnection()->write(getGad(), buf, 5);
+	BufferBuilder builder(5, DateObject::logger_m);
+	builder << 0 << (isWrite ? 0x80 : 0x40) << day_m << month_m;
+	builder << ((year_m >= 100 && year_m < 190) ? year_m-100 : year_m);
+
+    Services::instance()->getKnxConnection()->write(getGad(), builder.getBuffer(), 5);
 }
 
 void DateObject::setDate(int day, int month, int year)
@@ -1654,7 +1663,8 @@ void ValueObject::doWrite(const uint8_t* buf, int len, eibaddr_t src)
 
 void ValueObject::doSend(bool isWrite)
 {
-    uint8_t buf[4] = { 0, (isWrite ? 0x80 : 0x40), 0, 0 };
+	BufferBuilder builder(4, ValueObject::logger_m);
+	builder << 0 << (isWrite ? 0x80 : 0x40) << 0 << 0;
     int ex = 0;
     int m = (int)rint(getFloatValue() * 100);
     if (m < 0)
@@ -1666,7 +1676,7 @@ void ValueObject::doSend(bool isWrite)
             ex++;
         }
         m = -m;
-        buf[2] = ((m >> 8) & 0x07) | ((ex << 3) & 0x78) | (1 << 7);
+		builder.setValue(2, ((m >> 8) & 0x07) | ((ex << 3) & 0x78) | (1 << 7));
     }
     else
     {
@@ -1675,11 +1685,11 @@ void ValueObject::doSend(bool isWrite)
             m = m >> 1;
             ex++;
         }
-        buf[2] = ((m >> 8) & 0x07) | ((ex << 3) & 0x78);
+        builder.setValue(2, ((m >> 8) & 0x07) | ((ex << 3) & 0x78));
     }
-    buf[3] = (m & 0xff);
+    builder.setValue(3, m & 0xff);
 
-    Services::instance()->getKnxConnection()->write(getGad(), buf, 4);
+    Services::instance()->getKnxConnection()->write(getGad(), builder.getBuffer(), 4);
 }
 /*
 void ValueObjectImpl::setFloatValue(double value)
@@ -1744,15 +1754,16 @@ void ValueObject32::doWrite(const uint8_t* buf, int len, eibaddr_t src)
 
 void ValueObject32::doSend(bool isWrite)
 {
-    uint8_t buf[6] = { 0, (isWrite ? 0x80 : 0x40), 0, 0, 0, 0 };
+	BufferBuilder builder(6, ValueObject32::logger_m);
+	builder << 0 << (isWrite ? 0x80 : 0x40) << 0 << 0 << 0 << 0;
     convfloat tmp;
     tmp.fl = static_cast<float>(value_m);
-    buf[5] = static_cast<uint8_t>(tmp.u32 & 0x000000FF);
-    buf[4] = static_cast<uint8_t>((tmp.u32 & 0x0000FF00) >> 8);
-    buf[3] = static_cast<uint8_t>((tmp.u32 & 0x00FF0000) >> 16);
-    buf[2] = static_cast<uint8_t>((tmp.u32 & 0xFF000000) >> 24);
+    builder.setValue(5, (tmp.u32 & 0x000000FF));
+    builder.setValue(4, ((tmp.u32 & 0x0000FF00) >> 8));
+    builder.setValue(3, ((tmp.u32 & 0x00FF0000) >> 16));
+    builder.setValue(2, ((tmp.u32 & 0xFF000000) >> 24));
 
-    Services::instance()->getKnxConnection()->write(getGad(), buf, 6);
+    Services::instance()->getKnxConnection()->write(getGad(), builder.getBuffer(), 6);
 }
 
 UIntObjectValue::UIntObjectValue(const std::string& value)
@@ -1872,8 +1883,9 @@ void U8ImplObject::doWrite(const uint8_t* buf, int len, eibaddr_t src)
 
 void U8ImplObject::doSend(bool isWrite)
 {
-    uint8_t buf[3] = { 0, (isWrite ? 0x80 : 0x40), (getInt() & 0xff) };
-    Services::instance()->getKnxConnection()->write(getGad(), buf, 3);
+	BufferBuilder builder(3, U8ImplObject::logger_m);
+	builder << 0 << (isWrite ? 0x80 : 0x40) << (getInt() & 0xff);
+    Services::instance()->getKnxConnection()->write(getGad(), builder.getBuffer(), 3);
 }
 
 U8ObjectValue::U8ObjectValue(const std::string& value)
@@ -2061,8 +2073,10 @@ void HeatingModeObject::setValue(const std::string& value)
 
 Latin1CharObjectValue::Latin1CharObjectValue(const std::string& value)
 {
+	// Transcode passed value to latin1.
+	std::string latin1Value = StringObjectValue::transcode(value, StringObjectValue::getUTF8Encoding(), StringObjectValue::getLatin1Encoding());
     unsigned char chvalue;
-    std::istringstream val(value);
+    std::istringstream val(latin1Value);
     val >> chvalue;
 
     if ( val.fail() ||
@@ -2081,7 +2095,7 @@ std::string Latin1CharObjectValue::toString()
 {
     std::ostringstream out;
     out << (unsigned char)value_m;
-    return out.str();
+    return StringObjectValue::transcode(out.str(), StringObjectValue::getLatin1Encoding(), StringObjectValue::getUTF8Encoding());
 }
 
 Logger& Latin1CharObject::logger_m(Logger::getInstance("Latin1CharObject"));
@@ -2202,8 +2216,9 @@ void U16Object::doWrite(const uint8_t* buf, int len, eibaddr_t src)
 
 void U16Object::doSend(bool isWrite)
 {
-    uint8_t buf[4] = { 0, (isWrite ? 0x80 : 0x40), ((value_m & 0xff00)>>8), (value_m & 0xff) };
-    Services::instance()->getKnxConnection()->write(getGad(), buf, 4);
+	BufferBuilder builder(4, U16Object::logger_m);
+	builder << 0 << (isWrite ? 0x80 : 0x40) << ((value_m & 0xff00)>>8) << (value_m & 0xff);
+    Services::instance()->getKnxConnection()->write(getGad(), builder.getBuffer(), 4);
 }
 
 U32ObjectValue::U32ObjectValue(const std::string& value)
@@ -2259,8 +2274,9 @@ void U32Object::doWrite(const uint8_t* buf, int len, eibaddr_t src)
 
 void U32Object::doSend(bool isWrite)
 {
-    uint8_t buf[6] = { 0, (isWrite ? 0x80 : 0x40), ((value_m & 0xff000000)>>24), ((value_m & 0xff0000)>>16), ((value_m & 0xff00)>>8), (value_m & 0xff) };
-    Services::instance()->getKnxConnection()->write(getGad(), buf, 6);
+	BufferBuilder builder(6, U32Object::logger_m);
+    builder << 0 << (isWrite ? 0x80 : 0x40) << ((value_m & 0xff000000)>>24) << ((value_m & 0xff0000)>>16) << ((value_m & 0xff00)>>8) << (value_m & 0xff);
+    Services::instance()->getKnxConnection()->write(getGad(), builder.getBuffer(), 6);
 }
 
 RGBObjectValue::RGBObjectValue(const std::string& value)
@@ -2319,10 +2335,79 @@ void RGBObject::doWrite(const uint8_t* buf, int len, eibaddr_t src)
 
 void RGBObject::doSend(bool isWrite)
 {
-    uint8_t buf[5] = { 0, (isWrite ? 0x80 : 0x40), ((value_m & 0xff0000)>>16), ((value_m & 0xff00)>>8), (value_m & 0xff) };
-    Services::instance()->getKnxConnection()->write(getGad(), buf, 5);
+	BufferBuilder builder(5, RGBObject::logger_m);
+    builder << 0 << (isWrite ? 0x80 : 0x40) << ((value_m & 0xff0000)>>16) << ((value_m & 0xff00)>>8) << (value_m & 0xff);
+    Services::instance()->getKnxConnection()->write(getGad(), builder.getBuffer(), 5);
 }
 
+#ifdef STL_STREAM_SUPPORT_INT64
+RGBWObjectValue::RGBWObjectValue(const std::string& value)
+{
+    std::istringstream val(value);
+    val.setf(std::ios::hex, std::ios::basefield);
+    val >> RGBWObjectValue::value_m;
+
+    if ( val.fail() ||
+         val.peek() != std::char_traits<char>::eof()) // workaround for wrong val.eof() flag in uClibc++
+    {
+        std::stringstream msg;
+        msg << "RGBWObjectValue: Bad value: '" << value << "'" << std::endl;
+        throw ticpp::Exception(msg.str());
+    }
+}
+
+std::string RGBWObjectValue::toString()
+{
+    std::ostringstream out;
+    out.setf(std::ios::hex, std::ios::basefield);
+    out.fill('0');
+    out << std::setw(8) << RGBWObjectValue::value_m;
+    return out.str();
+}
+
+Logger& RGBWObject::logger_m(Logger::getInstance("RGBWObject"));
+
+RGBWObject::RGBWObject()
+{}
+
+RGBWObject::~RGBWObject()
+{}
+
+ObjectValue* RGBWObject::createObjectValue(const std::string& value)
+{
+    return new RGBWObjectValue(value);
+}
+
+void RGBWObject::setValue(const std::string& value)
+{
+    RGBWObjectValue val(value);
+    Object::setValue(&val);
+}
+
+void RGBWObject::doWrite(const uint8_t* buf, int len, eibaddr_t src)
+{
+    unsigned int newValue;
+    newValue = (buf[2]<<24) | (buf[3]<<16) | (buf[4]<<8) | buf[5];
+    if (forceUpdate() || newValue != RGBWObjectValue::value_m)
+    {
+        RGBWObjectValue::value_m = newValue;
+        onUpdate();
+    }
+}
+
+void RGBWObject::doSend(bool isWrite)
+{
+	BufferBuilder builder(8, RGBWObject::logger_m);
+    builder << 0 << (isWrite ? 0x80 : 0x40)
+    << ((RGBWObjectValue::value_m & 0xff000000)>>24)
+    << ((RGBWObjectValue::value_m & 0xff0000)>>16)
+    << ((RGBWObjectValue::value_m & 0xff00)>>8)
+    << (RGBWObjectValue::value_m & 0xff)
+    << 0x00 << 0x0f;
+    Services::instance()->getKnxConnection()->write(getGad(), builder.getBuffer(), 8);
+}
+#endif
+
 IntObjectValue::IntObjectValue(const std::string& value)
 {
     std::istringstream val(value);
@@ -2479,8 +2564,9 @@ void S8Object::doWrite(const uint8_t* buf, int len, eibaddr_t src)
 
 void S8Object::doSend(bool isWrite)
 {
-    uint8_t buf[3] = { 0, (isWrite ? 0x80 : 0x40), (value_m & 0xff) };
-    Services::instance()->getKnxConnection()->write(getGad(), buf, 3);
+	BufferBuilder builder(3, S8Object::logger_m);
+    builder << 0 << (isWrite ? 0x80 : 0x40) << (value_m & 0xff);
+    Services::instance()->getKnxConnection()->write(getGad(), builder.getBuffer(), 3);
 }
 
 S16ObjectValue::S16ObjectValue(const std::string& value)
@@ -2540,8 +2626,9 @@ void S16Object::doWrite(const uint8_t* buf, int len, eibaddr_t src)
 
 void S16Object::doSend(bool isWrite)
 {
-    uint8_t buf[4] = { 0, (isWrite ? 0x80 : 0x40), ((value_m & 0xff00)>>8), (value_m & 0xff) };
-    Services::instance()->getKnxConnection()->write(getGad(), buf, 4);
+	BufferBuilder builder(4, S16Object::logger_m);
+    builder << 0 << (isWrite ? 0x80 : 0x40) << ((value_m & 0xff00)>>8) << (value_m & 0xff);
+    Services::instance()->getKnxConnection()->write(getGad(), builder.getBuffer(), 4);
 }
 
 S32ObjectValue::S32ObjectValue(const std::string& value)
@@ -2597,8 +2684,9 @@ void S32Object::doWrite(const uint8_t* buf, int len, eibaddr_t src)
 
 void S32Object::doSend(bool isWrite)
 {
-    uint8_t buf[6] = { 0, (isWrite ? 0x80 : 0x40), ((value_m & 0xff000000)>>24), ((value_m & 0xff0000)>>16), ((value_m & 0xff00)>>8), (value_m & 0xff) };
-    Services::instance()->getKnxConnection()->write(getGad(), buf, 6);
+	BufferBuilder builder(6, S32Object::logger_m);
+    builder << 0 << (isWrite ? 0x80 : 0x40) << ((value_m & 0xff000000)>>24) << ((value_m & 0xff0000)>>16) << ((value_m & 0xff00)>>8) << (value_m & 0xff);
+    Services::instance()->getKnxConnection()->write(getGad(), builder.getBuffer(), 6);
 }
 
 #ifdef STL_STREAM_SUPPORT_INT64
@@ -2723,9 +2811,10 @@ void S64Object::doWrite(const uint8_t* buf, int len, eibaddr_t src)
 
 void S64Object::doSend(bool isWrite)
 {
-    uint8_t buf[10] = { 0, (isWrite ? 0x80 : 0x40), ((value_m & 0xff00000000000000LL)>>56), ((value_m & 0xff000000000000LL)>>48), ((value_m & 0xff0000000000LL)>>40), ((value_m & 0xff00000000LL)>>32),
-                                                    ((value_m & 0xff000000LL)>>24), ((value_m & 0xff0000LL)>>16), ((value_m & 0xff00LL)>>8), (value_m & 0xffLL) };
-    Services::instance()->getKnxConnection()->write(getGad(), buf, 10);
+	BufferBuilder builder(10, S64Object::logger_m);
+    builder << 0 << (isWrite ? 0x80 : 0x40) << ((value_m & 0xff00000000000000LL)>>56) << ((value_m & 0xff000000000000LL)>>48) << ((value_m & 0xff0000000000LL)>>40) << ((value_m & 0xff00000000LL)>>32)
+                                                    << ((value_m & 0xff000000LL)>>24) << ((value_m & 0xff0000LL)>>16) << ((value_m & 0xff00LL)>>8) << (value_m & 0xffLL);
+    Services::instance()->getKnxConnection()->write(getGad(), builder.getBuffer(), 10);
 }
 #endif
 
@@ -2815,6 +2904,45 @@ bool StringObjectValue::set(double value)
     return false;
 }
 
+const std::string &StringObjectValue::getLatin1Encoding()
+{
+	static const std::string encoding("ISO-8859-1"); // As iconv expects it (see https://www.gnu.org/savannah-checkouts/gnu/libiconv/documentation/libiconv-1.13/iconv_open.3.html).
+	return encoding;
+}
+
+const std::string &StringObjectValue::getUTF8Encoding()
+{
+	static const std::string encoding("UTF-8"); // As iconv expects it (see https://www.gnu.org/savannah-checkouts/gnu/libiconv/documentation/libiconv-1.13/iconv_open.3.html).
+	return encoding;
+}
+
+std::string StringObjectValue::transcode(const std::string &source, const std::string &sourceEncoding, const std::string &targetEncoding)
+{
+	if (source.empty()) return source; // iconv seems to not support empty strings.
+
+	// Ask iconv to use replacement chars that look like the original ones for characters
+	// that are unavailable in target encoding.
+	iconv_t conversionDescriptor = iconv_open((targetEncoding + "//TRANSLIT").c_str(), sourceEncoding.c_str());
+	char cSource[source.size()];
+	memcpy(cSource, source.c_str(), source.size() + 1);
+	ICONV_CONST char *sourceStart = &cSource[0];
+	size_t sourceLength = source.size();
+	const size_t targetLength = source.size() * 5; // Should be pretty enough even in worst cases.
+	char targetChars[targetLength];
+	char *targetCharPointer = &targetChars[0];
+	size_t targetSizeLeft = targetLength;
+	size_t res = iconv(conversionDescriptor, &sourceStart, &sourceLength, &targetCharPointer, &targetSizeLeft);
+	if(res == static_cast<size_t>(-1))
+	{
+		logger_m.errorStream() << "Transcoding of " << source << " from " << sourceEncoding << " to " << targetEncoding << " failed." << endlog;
+		return source;
+	}
+
+	std::string transcodedString(targetChars, targetLength - targetSizeLeft);
+	return transcodedString;
+}
+
+
 Logger& StringObject::logger_m(Logger::getInstance("StringObject"));
 
 StringObject::StringObject()
@@ -2873,7 +3001,7 @@ void StringObject::setStringValue(const std::string& value)
 
 String14ObjectValue::String14ObjectValue(const std::string& value): StringObjectValue(value)
 {
-    if ( value.length() > 14)
+    if (transcode(value, getUTF8Encoding(), getLatin1Encoding()).length() > 14)
     {
         std::stringstream msg;
         msg << "String14ObjectValue: Bad value (too long): '" << value << "'" << std::endl;
@@ -2931,6 +3059,8 @@ void String14Object::doWrite(const uint8_t* buf, int len, eibaddr_t src)
     std::string value;
     for(int j=2; j<len && buf[j]!=0; j++)
         value.push_back(buf[j]);
+
+	value = transcode(value, getLatin1Encoding(), getUTF8Encoding());
     String14ObjectValue val(value);
 
     if (set(&val) || forceUpdate())
@@ -2944,8 +3074,9 @@ void String14Object::doSend(bool isWrite)
     memset(buf,0,sizeof(buf));
     buf[1] = (isWrite ? 0x80 : 0x40);
     // Convert to hex
-    for(uint j=0;j<value_m.size();j++)
-        buf[j+2] = static_cast<uint8_t>(value_m[j]);
+	std::string latin1Value = transcode(value_m, getUTF8Encoding(), getLatin1Encoding());
+    for(uint j=0;j<latin1Value.size();j++)
+        buf[j+2] = static_cast<uint8_t>(latin1Value[j]);
 
     Services::instance()->getKnxConnection()->write(getGad(), buf, sizeof(buf));
 }
@@ -3217,3 +3348,14 @@ std::list<Object*> ObjectController::getObjects()
     }
     return objects;
 }
+
+Object::BufferBuilder::BufferBuilder(int size, Logger &logger)
+ : buffer_m(NULL), index_m(0), size_m(size), logger_m(logger)
+{
+	buffer_m = new uint8_t[size];
+}
+
+Object::BufferBuilder::~BufferBuilder()
+{
+	delete[] buffer_m;
+}
