@@ -28,6 +28,8 @@
 #include "smsgateway.h"
 #include "emailgateway.h"
 #include "knxconnection.h"
+#include "persistentstorage.h"
+#include "suncalc.h"
 
 
 class Services
@@ -51,6 +53,11 @@ public:
     EmailGateway* getEmailGateway() { return &emailGateway_m; };
     TimerManager* getTimerManager() { return &timers_m; };
     ExceptionDays* getExceptionDays() { return &exceptionDays_m; };
+    PersistentStorage* getPersistentStorage() { return persistentStorage_m; };
+    LocationInfo* getLocationInfo() { return &locationInfo_m; };
+    void setConfigFile(const char* filename) { if (filename) configFile_m = filename; };
+    std::string getConfigFile() { return configFile_m; };
+    void createDefault();
 
 private:
     Services();
@@ -59,11 +66,15 @@ private:
     static Services* instance_m;
 
     XmlServer *xmlServer_m;
+    PersistentStorage *persistentStorage_m;
     TimerManager timers_m;
     SmsGateway smsGateway_m;
     EmailGateway emailGateway_m;
     KnxConnection knxConnection_m;
     ExceptionDays exceptionDays_m;
+    LocationInfo locationInfo_m;
+    
+    std::string configFile_m;
 };
 
 #endif
