@@ -29,6 +29,88 @@
 #include "ticpp.h"
 #include "objectcontroller.h"
 
+class DateTime
+{
+public:
+	enum FieldType
+	{
+		Invalid = -1,
+		Year = 0,
+		Month = 1,
+		Day = 2,
+		Hour = 3,
+		Minute = 4,
+	};
+
+	enum ResolutionResult
+	{
+		Resolution_Resolved,
+		Resolution_Unresolved,
+		Resolution_Impossible
+	};
+
+private:
+	enum ProjectionResult
+	{
+		Projection_Succeeded,
+		Projection_Failed,
+		Projection_Changed,
+		Projection_Impossible
+	};
+
+public:
+	DateTime(const tm *time);
+
+public:
+	int getYear() const { return getField(Year); }
+	void setYear(int year) { setField(Year, year, true); }
+	int getMonth() const { return getField(Month); }
+	void setMonth(int month) { setField(Month, month, true); }
+	int getDay() const { return getField(Day); }
+	void setDay(int day) { setField(Day, day, true); }
+	int getHour() const { return getField(Hour); }
+	void setHour(int hour) { setField(Hour, hour, true); }
+	int getMinute() const { return getField(Minute); }
+	void setMinute(int min) { setField(Minute, min, true); }
+	int getWeekdays() const { return weekdays_m; }
+	void setWeekdays(int wd) { weekdays_m = wd; }
+
+	int getField(FieldType field) const;
+
+	void setField(FieldType field, int value, bool fixesIfChanged);
+
+	bool isFieldFixed(FieldType detail) const;
+
+	bool isFieldFree(FieldType detail) const;
+
+	bool tryIncreaseClosestGreaterFreeField(FieldType current);
+
+	int increaseField(FieldType fieldId, bool fixes);
+
+	time_t getTime(tm *outBrokenDownTime = NULL) const;
+
+	/** Attempts to ensure constraints are met and adjusts free fields if
+	 * required so that the date/time represented by this object satisfies
+	 * the various constraints and comes after current date/time. */
+	ResolutionResult tryResolve(const DateTime &current, FieldType from, FieldType to);
+
+	bool operator>(const DateTime &other) const;
+
+	bool operator<(const DateTime &other) const;
+
+private:
+	bool tryResolveUnprojected(const DateTime &current, FieldType from, FieldType to);
+	bool tryResolveWithoutWeekdays(const DateTime &current, FieldType from, FieldType to);
+	ProjectionResult projectOnActualCalendar();
+	void resetFieldIfFree(FieldType field, bool recurses);
+	bool isCompatibleWithWeekDays() const;
+
+private:
+   	int fields_m[5];
+	int freeFields_m;
+	int weekdays_m;
+};
+
 class TimerTask
 {
 public:
@@ -62,7 +144,7 @@ public:
     };
 
     TimeSpec();
-    TimeSpec(int min, int hour, int mday, int mon, int year);
+    TimeSpec(int min, int hour, int mday, int mon, int year, int offset=0);
     TimeSpec(int min, int hour, int wdays=All, ExceptionDays exception=DontCare);
     virtual ~TimeSpec() {};
 
@@ -72,16 +154,33 @@ public:
     virtual void importXml(ticpp::Element* pConfig);
     virtual void exportXml(ticpp::Element* pConfig);
 
-    virtual void getData(int *min, int *hour, int *mday, int *mon, int *year, int *wdays, ExceptionDays *exception, const struct tm * timeinfo);
-    virtual bool adjustTime(struct tm * timeinfo) { return false; };
-protected:
-    //		int sec_m;
+	bool isValid() const;
+	int getDayOfMonth() const {return mday_m;}
+	void setDayOfMonth(int value) {mday_m = value;}
+	int getMonth() const {return mon_m + 1;}
+	void setMonth(int value) {mon_m = value - 1;}
+	int getYear() const {return year_m + 1900;}
+	void setYear(int value) {year_m = value - 1900;}
+	int getHour() const {return hour_m;}
+	void setHour(int value) {hour_m = value;}
+	int getMinute() const {return min_m;}
+	void setMinute(int value) {min_m = value;}
+
+    virtual void getDay(const tm &current, int &mday, int &mon, int &year, int &wdays) const;
+    virtual void getTime(int mday, int mon, int year, int &min, int &hour) const;
+	int getOffsetInSeconds() const { return offset_m; }
+    ExceptionDays getExceptions() const { return exception_m; }
+	void checkIsValid() const;
+
+private:
     int min_m;
     int hour_m;
     int mday_m;
     int mon_m;
     int year_m;
     int wdays_m;
+	/** Offset in seconds. */
+    int offset_m;
     ExceptionDays exception_m;
 
 };
@@ -90,17 +189,21 @@ class VariableTimeSpec : public TimeSpec
 {
 public:
     VariableTimeSpec(ChangeListener* cl);
+    VariableTimeSpec(ChangeListener* cl, int min, int hour, int mday, int mon, int year, int offset);
     virtual ~VariableTimeSpec();
 
     virtual void importXml(ticpp::Element* pConfig);
     virtual void exportXml(ticpp::Element* pConfig);
 
-    virtual void getData(int *min, int *hour, int *mday, int *mon, int *year, int *wdays, ExceptionDays *exception, const struct tm * timeinfo);
+    virtual void getDay(const tm &current, int &mday, int &mon, int &year, int &wdays) const;
+    virtual void getTime(int mday, int mon, int year, int &min, int &hour) const;
+
+private:
+	void getDataFromObject(int &min, int &hour, int &mday, int &mon, int &year, int &wdays) const;
 protected:
     TimeObject* time_m;
     DateObject* date_m;
     ChangeListener* cl_m;
-    int offset_m;
 };
 
 class PeriodicTask : public TimerTask, public ChangeListener
@@ -129,6 +232,9 @@ protected:
     time_t findNext(time_t start, TimeSpec* next);
     time_t mktimeNoDst(struct tm * timeinfo);
     static Logger& logger_m;
+
+private:
+	time_t goToNextDayAndFindNext(const DateTime &curent, TimeSpec *next);
 };
 
 class FixedTimeTask : public TimerTask
