@@ -1060,6 +1060,11 @@ TiXmlDocument::TiXmlDocument( const TiXmlDocument& copy ) : TiXmlNode( TiXmlNode
 }
 
 
+TiXmlDocument::~TiXmlDocument()
+{
+}
+
+
 void TiXmlDocument::operator=( const TiXmlDocument& copy )
 {
 	Clear();
@@ -1128,20 +1133,89 @@ bool TiXmlDocument::GetAsCharBuffer( char* buffer, size_t bufferSize )
 /** Load the document from a string. */
 bool TiXmlDocument::LoadFromString( const char* document, TiXmlEncoding encoding )
 {
-	// Copy string to temporary FILE buffer
-	FILE* fileBuf = tmpfile();
-	size_t length = strlen( document );
-	size_t written = fwrite( document, sizeof( char ), length, fileBuf );
-	if ( (size_t)written < length )
-	{
-		return false;
-	}
-	rewind( fileBuf );
+    // Delete the existing data:
+    Clear();
+    location.Clear();
 
-	// Load FILE into TiXmlDocument
-	bool result = LoadFile( fileBuf, encoding );
-	fclose( fileBuf );
-	return result;
+    // Get the file size, so we can pre-allocate the string. HUGE speed impact.
+    size_t length = strlen( document );
+
+    // Strange case, but good to handle up front.
+    if ( length == 0 )
+    {
+        SetError( TIXML_ERROR_DOCUMENT_EMPTY, 0, 0, TIXML_ENCODING_UNKNOWN );
+        return false;
+    }
+
+    // If we have a file, assume it is all one big XML file, and read it in.
+    // The document parser may decide the document ends sooner than the entire file, however.
+    TIXML_STRING data;
+    data.reserve( length );
+
+    // Subtle bug here. TinyXml did use fgets. But from the XML spec:
+    // 2.11 End-of-Line Handling
+    // <snip>
+    // <quote>
+    // ...the XML processor MUST behave as if it normalized all line breaks in external
+    // parsed entities (including the document entity) on input, before parsing, by translating
+    // both the two-character sequence #xD #xA and any #xD that is not followed by #xA to
+    // a single #xA character.
+    // </quote>
+    //
+    // It is not clear fgets does that, and certainly isn't clear it works cross platform.
+    // Generally, you expect fgets to translate from the convention of the OS to the c/unix
+    // convention, and not work generally.
+
+    const char* buf = document;
+    const char* lastPos = buf;
+    const char* p = buf;
+
+    while( *p ) {
+        assert( p < (buf+length) );
+        if ( *p == 0xa ) {
+            // Newline character. No special rules for this. Append all the characters
+            // since the last string, and include the newline.
+            data.append( lastPos, (p-lastPos+1) );  // append, include the newline
+            ++p;                                    // move past the newline
+            lastPos = p;                            // and point to the new buffer (may be 0)
+            assert( p <= (buf+length) );
+        }
+        else if ( *p == 0xd ) {
+            // Carriage return. Append what we have so far, then
+            // handle moving forward in the buffer.
+            if ( (p-lastPos) > 0 ) {
+                data.append( lastPos, p-lastPos );  // do not add the CR
+            }
+            data += (char)0xa;                      // a proper newline
+
+            if ( *(p+1) == 0xa ) {
+                // Carriage return - new line sequence
+                p += 2;
+                lastPos = p;
+                assert( p <= (buf+length) );
+            }
+            else {
+                // it was followed by something else...that is presumably characters again.
+                ++p;
+                lastPos = p;
+                assert( p <= (buf+length) );
+            }
+        }
+        else {
+            ++p;
+        }
+    }
+    // Handle any left over characters.
+    if ( p-lastPos ) {
+        data.append( lastPos, p-lastPos );
+    }
+
+    Parse( data.c_str(), 0, encoding );
+
+    if (  Error() )
+        return false;
+    else
+        return true;
 }
 
 bool TiXmlDocument::LoadFile( const char* filename, TiXmlEncoding encoding )
@@ -1180,10 +1254,6 @@ bool TiXmlDocument::LoadFile( FILE* file, TiXmlEncoding encoding )
 		return false;
 	}
 
-	// Delete the existing data:
-	Clear();
-	location.Clear();
-
 	// Get the file size, so we can pre-allocate the string. HUGE speed impact.
 	long length = 0;
 	fseek( file, 0, SEEK_END );
@@ -1197,32 +1267,6 @@ bool TiXmlDocument::LoadFile( FILE* file, TiXmlEncoding encoding )
 		return false;
 	}
 
-	// If we have a file, assume it is all one big XML file, and read it in.
-	// The document parser may decide the document ends sooner than the entire file, however.
-	TIXML_STRING data;
-	data.reserve( length );
-
-	// Subtle bug here. TinyXml did use fgets. But from the XML spec:
-	// 2.11 End-of-Line Handling
-	// <snip>
-	// <quote>
-	// ...the XML processor MUST behave as if it normalized all line breaks in external
-	// parsed entities (including the document entity) on input, before parsing, by translating
-	// both the two-character sequence #xD #xA and any #xD that is not followed by #xA to
-	// a single #xA character.
-	// </quote>
-	//
-	// It is not clear fgets does that, and certainly isn't clear it works cross platform.
-	// Generally, you expect fgets to translate from the convention of the OS to the c/unix
-	// convention, and not work generally.
-
-	/*
-	while( fgets( buf, sizeof(buf), file ) )
-	{
-		data += buf;
-	}
-	*/
-
 	char* buf = new char[ length+1 ];
 	buf[0] = 0;
 
@@ -1232,58 +1276,13 @@ bool TiXmlDocument::LoadFile( FILE* file, TiXmlEncoding encoding )
 		return false;
 	}
 
-	const char* lastPos = buf;
-	const char* p = buf;
-
 	buf[length] = 0;
-	while( *p ) {
-		assert( p < (buf+length) );
-		if ( *p == 0xa ) {
-			// Newline character. No special rules for this. Append all the characters
-			// since the last string, and include the newline.
-			data.append( lastPos, (p-lastPos+1) );	// append, include the newline
-			++p;									// move past the newline
-			lastPos = p;							// and point to the new buffer (may be 0)
-			assert( p <= (buf+length) );
-		}
-		else if ( *p == 0xd ) {
-			// Carriage return. Append what we have so far, then
-			// handle moving forward in the buffer.
-			if ( (p-lastPos) > 0 ) {
-				data.append( lastPos, p-lastPos );	// do not add the CR
-			}
-			data += (char)0xa;						// a proper newline
+	bool result = LoadFromString( buf, encoding );
 
-			if ( *(p+1) == 0xa ) {
-				// Carriage return - new line sequence
-				p += 2;
-				lastPos = p;
-				assert( p <= (buf+length) );
-			}
-			else {
-				// it was followed by something else...that is presumably characters again.
-				++p;
-				lastPos = p;
-				assert( p <= (buf+length) );
-			}
-		}
-		else {
-			++p;
-		}
-	}
-	// Handle any left over characters.
-	if ( p-lastPos ) {
-		data.append( lastPos, p-lastPos );
-	}
 	delete [] buf;
 	buf = 0;
 
-	Parse( data.c_str(), 0, encoding );
-
-	if (  Error() )
-        return false;
-    else
-		return true;
+    return result;
 }
 
 
